/*
 ## Cypress FX3 Camera Kit source file (sensor.c)
 ## ===========================
 ##
 ##  Copyright Cypress Semiconductor Corporation, 2010-2012,
 ##  All Rights Reserved
 ##  UNPUBLISHED, LICENSED SOFTWARE.
 ##
 ##  CONFIDENTIAL AND PROPRIETARY INFORMATION
 ##  WHICH IS THE PROPERTY OF CYPRESS.
 ##
 ##  Use of this file is governed
 ##  by the license agreement included in the file
 ##
 ##     <install>/license/license.txt
 ##
 ##  where <install> is the Cypress software
 ##  installation root directory path.
 ##
 ## ===========================
*/

/* This file implements the I2C based driver for the MT9M114 image sensor used
   in the FX3 HD 720p camera kit.

   Please refer to the Aptina MT9M114 sensor datasheet for the details of the
   I2C commands used to configure the sensor.
 */

#include <cyu3system.h>
#include <cyu3os.h>
#include <cyu3dma.h>
#include <cyu3error.h>
#include <cyu3uart.h>
#include <cyu3i2c.h>
#include <cyu3types.h>
#include <cyu3gpio.h>
#include <cyu3utils.h>
#include "sensor.h"
#include "ov_5642_conf.h"

/* This function inserts a delay between successful I2C transfers to prevent
   false errors due to the slave being busy.
 */
static void
SensorI2CAccessDelay (
        CyU3PReturnStatus_t status)
{
    /* Add a 10us delay if the I2C operation that preceded this call was successful. */
    if (status == CY_U3P_SUCCESS)
        CyU3PBusyWait (10);
}

/* Write to an I2C slave with two bytes of data. */
CyU3PReturnStatus_t
SensorWrite2B (
        uint8_t slaveAddr,
        uint8_t highAddr,
        uint8_t lowAddr,
        uint8_t highData,
        uint8_t lowData)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t  preamble;
    uint8_t buf[2];

    /* Validate the I2C slave address. */
    if ((slaveAddr != SENSOR_ADDR_WR) && (slaveAddr != I2C_MEMORY_ADDR_WR))
    {
        CyU3PDebugPrint (4, "I2C Slave address is not valid!\n");
        return 1;
    }

    /* Set the parameters for the I2C API access and then call the write API. */
    preamble.buffer[0] = slaveAddr;
    preamble.buffer[1] = highAddr;
    preamble.buffer[2] = lowAddr;
    preamble.length    = 3;             /*  Three byte preamble. */
    preamble.ctrlMask  = 0x0000;        /*  No additional start and stop bits. */

    buf[0] = highData;
    buf[1] = lowData;

    apiRetStatus = CyU3PI2cTransmitBytes (&preamble, buf, 2, 0);
    SensorI2CAccessDelay (apiRetStatus);

    return apiRetStatus;
}

CyU3PReturnStatus_t
SensorWrite1B (
        uint8_t slaveAddr,
        uint8_t highAddr,
        uint8_t lowAddr,
        uint8_t Data)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t  preamble;
    uint8_t buf[1];

    /* Validate the I2C slave address. */
    if ((slaveAddr != SENSOR_ADDR_WR) && (slaveAddr != I2C_MEMORY_ADDR_WR))
    {
        CyU3PDebugPrint (4, "I2C Slave address is not valid!\n");
        return 1;
    }

    /* Set the parameters for the I2C API access and then call the write API. */
    preamble.buffer[0] = slaveAddr;
    preamble.buffer[1] = highAddr;
    preamble.buffer[2] = lowAddr;
    preamble.length    = 3;             /*  Three byte preamble. */
    preamble.ctrlMask  = 0x0000;        /*  No additional start and stop bits. */

    buf[0] = Data;

    apiRetStatus = CyU3PI2cTransmitBytes (&preamble, buf, 1, 0);
    SensorI2CAccessDelay (apiRetStatus);

    return apiRetStatus;
}

CyU3PReturnStatus_t
SensorWrite (
        uint8_t slaveAddr,
        uint8_t highAddr,
        uint8_t lowAddr,
        uint8_t count,
        uint8_t *buf)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t preamble;

    /* Validate the I2C slave address. */
    if ((slaveAddr != SENSOR_ADDR_WR) && (slaveAddr != I2C_MEMORY_ADDR_WR))
    {
        CyU3PDebugPrint (4, "I2C Slave address is not valid!\n");
        return 1;
    }

    if (count > 64)
    {
        CyU3PDebugPrint (4, "ERROR: SensorWrite count > 64\n");
        return 1;
    }

    /* Set up the I2C control parameters and invoke the write API. */
    preamble.buffer[0] = slaveAddr;
    preamble.buffer[1] = highAddr;
    preamble.buffer[2] = lowAddr;
    preamble.length    = 3;
    preamble.ctrlMask  = 0x0000;

    apiRetStatus = CyU3PI2cTransmitBytes (&preamble, buf, count, 0);
    SensorI2CAccessDelay (apiRetStatus);

    return apiRetStatus;
}

CyU3PReturnStatus_t
SensorRead2B (
        uint8_t slaveAddr,
        uint8_t highAddr,
        uint8_t lowAddr,
        uint8_t *buf)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t preamble;

    if ((slaveAddr != SENSOR_ADDR_RD) && (slaveAddr != I2C_MEMORY_ADDR_RD))
    {
        CyU3PDebugPrint (4, "I2C Slave address is not valid!\n");
        return 1;
    }

    preamble.buffer[0] = slaveAddr & I2C_SLAVEADDR_MASK;        /*  Mask out the transfer type bit. */
    preamble.buffer[1] = highAddr;
    preamble.buffer[2] = lowAddr;
    preamble.buffer[3] = slaveAddr;
    preamble.length    = 4;
    preamble.ctrlMask  = 0x0004;                                /*  Send start bit after third byte of preamble. */

    apiRetStatus = CyU3PI2cReceiveBytes (&preamble, buf, 2, 0);
    SensorI2CAccessDelay (apiRetStatus);

    return apiRetStatus;
}

CyU3PReturnStatus_t
SensorRead1B (
        uint8_t slaveAddr,
        uint16_t highAddr,
        uint16_t lowAddr,
        uint8_t *buf)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t preamble;

    if ((slaveAddr != SENSOR_ADDR_RD) && (slaveAddr != I2C_MEMORY_ADDR_RD))
    {
        CyU3PDebugPrint (4, "I2C Slave address is not valid!\n");
        return 1;
    }

    preamble.buffer[0] = slaveAddr & I2C_SLAVEADDR_MASK;        /*  Mask out the transfer type bit. */
    preamble.buffer[1] = highAddr;
    preamble.buffer[2] = lowAddr;
    preamble.buffer[3] = slaveAddr;
    preamble.length    = 4;
    preamble.ctrlMask  = 0x0004;                                /*  Send start bit after third byte of preamble. */

    apiRetStatus = CyU3PI2cReceiveBytes (&preamble, buf, 1, 0);
    SensorI2CAccessDelay (apiRetStatus);

    return apiRetStatus;
}

CyU3PReturnStatus_t
SensorRead (
        uint8_t slaveAddr,
        uint8_t highAddr,
        uint8_t lowAddr,
        uint8_t count,
        uint8_t *buf)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t preamble;

    /* Validate the parameters. */
    if ((slaveAddr != SENSOR_ADDR_RD) && (slaveAddr != I2C_MEMORY_ADDR_RD))
    {
        CyU3PDebugPrint (4, "I2C Slave address is not valid!\n");
        return 1;
    }
    if ( count > 64 )
    {
        CyU3PDebugPrint (4, "ERROR: SensorWrite count > 64\n");
        return 1;
    }

    preamble.buffer[0] = slaveAddr & I2C_SLAVEADDR_MASK;        /*  Mask out the transfer type bit. */
    preamble.buffer[1] = highAddr;
    preamble.buffer[2] = lowAddr;
    preamble.buffer[3] = slaveAddr;
    preamble.length    = 4;
    preamble.ctrlMask  = 0x0004;                                /*  Send start bit after third byte of preamble. */

    apiRetStatus = CyU3PI2cReceiveBytes (&preamble, buf, count, 0);
    SensorI2CAccessDelay (apiRetStatus);

    return apiRetStatus;
}

/*
 * Reset the MT9M114 sensor using GPIO.
 */
void
SensorReset (
        void)
{
    CyU3PReturnStatus_t apiRetStatus;

    /* Drive the GPIO low to reset the sensor. */
    apiRetStatus = CyU3PGpioSetValue (SENSOR_RESET_GPIO, CyFalse);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "GPIO Set Value Error, Error Code = %d\n", apiRetStatus);
        return;
    }

    /* Wait for some time to allow proper reset. */
    CyU3PThreadSleep (10);

    /* Drive the GPIO high to bring the sensor out of reset. */
    apiRetStatus = CyU3PGpioSetValue (SENSOR_RESET_GPIO, CyTrue);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "GPIO Set Value Error, Error Code = %d\n", apiRetStatus);
        return;
    }

    /* Delay the allow the sensor to power up. */
    CyU3PThreadSleep (10);
    return;
}

/* MT9M114 sensor initialization sequence.
   Adapted from MT9M114-REV2.ini file provided by Aptina.

   Step1_Post_Reset       : Post-Reset Settings
   Step2_PLL_Timing       : PLL and Timing settings
   Step3_Recommended      : Patch, Errata and Sensor optimization Setting
   Step4_APGA             : APGA
   Step5_AWB_CCM          : AWB & CCM
   Step7_PIPE_Preference  : Color Pipe preference settings
   Step8_Features         : Ports, special features etc.
   Select Video Resolution
   SensorChangeConfig     : Update sensor configuration.
*/
void
SensorInit (
        void)
{
	uint16_t count;
	uint8_t a[1];
    if (SensorI2cBusTest () != CY_U3P_SUCCESS)        /* Verify that the sensor is connected. */
    {
        CyU3PDebugPrint (4, "Error: Reading Sensor ID failed!\r\n");
        return;
    }
    SensorWrite1B(SENSOR_ADDR_WR,0x3f,0x00,0x01);
    CyU3PThreadSleep (10);
    SensorWrite1B(SENSOR_ADDR_WR,0x30,0x08,0x82);
    CyU3PThreadSleep (10);


    SensorRead1B (SENSOR_ADDR_RD,0x41,0x00,a);
    CyU3PDebugPrint (4, "0x4100: %d\r\n",a[0]);

    for (count=0;count<460;count++)
    	SensorWrite1B(SENSOR_ADDR_WR, (basic_conf[2*count]&0xff00)>>8, basic_conf[2*count]&0xff, basic_conf[2*count+1]&0xff);
    for (count=0;count<37;count++)
        SensorWrite1B(SENSOR_ADDR_WR, (format_conf[2*count]&0xff00)>>8, format_conf[2*count]&0xff, format_conf[2*count+1]&0xff);
    for (count=0;count<136;count++)
        SensorWrite1B(SENSOR_ADDR_WR, (lens_corr_conf[2*count]&0xff00)>>8, lens_corr_conf[2*count]&0xff, lens_corr_conf[2*count+1]&0xff);
    for (count=0;count<31;count++)
        SensorWrite1B(SENSOR_ADDR_WR, (AWB_conf[2*count]&0xff00)>>8, AWB_conf[2*count]&0xff, AWB_conf[2*count+1]&0xff);
    for (count=0;count<21;count++)
        SensorWrite1B(SENSOR_ADDR_WR, (color_martix_conf[2*count]&0xff00)>>8, color_martix_conf[2*count]&0xff, color_martix_conf[2*count+1]&0xff);
    for (count=0;count<56;count++)
        SensorWrite1B(SENSOR_ADDR_WR, (Gamma_conf[2*count]&0xff00)>>8, Gamma_conf[2*count]&0xff, Gamma_conf[2*count+1]&0xff);
    for (count=0;count<6;count++)
        SensorWrite1B(SENSOR_ADDR_WR, (AE_conf[2*count]&0xff00)>>8, AE_conf[2*count]&0xff, AE_conf[2*count+1]&0xff);


    SensorRead1B (SENSOR_ADDR_RD,0x47,0x40,a);
        CyU3PDebugPrint (4, "0x4740: %d\r\n",a[0]);
}

/*
 * Verify that the sensor can be accessed over the I2C bus from FX3.
 */
uint8_t
SensorI2cBusTest (
        void)
{
    /* The sensor ID register can be read here to verify sensor connectivity. */
    uint8_t buf[2];

    /* Reading sensor ID */
    if (SensorRead1B (SENSOR_ADDR_RD, 0x30, 0x0b, buf) == CY_U3P_SUCCESS)
    {
        if (buf[0] == 0x42)
        {
            return CY_U3P_SUCCESS;
        }
    }
    return 1;
}

/*
   The procedure is adapted from Aptina's sensor initialization scripts. Please
   refer to the MT9M114 sensor datasheet for details.
 */
void
SensorScaling_VGA (
        void)
{

    SensorChangeConfig();
}

/*
   The procedure is adapted from Aptina's sensor initialization scripts. Please
   refer to the MT9M114 sensor datasheet for details.
 */
void
SensorScaling_HD720p_30fps (
        void)
{


    SensorChangeConfig();
}

/*
   The procedure is adapted from Aptina's sensor initialization scripts. Please
   refer to the MT9M114 sensor datasheet for details.
 */
void
SensorChangeConfig (
        void)
{

}

/*
   Get the current brightness setting from the MT9M114 sensor.
 */
uint8_t
SensorGetBrightness (
        void)
{
	uint8_t brightness[1];
	SensorRead1B (SENSOR_ADDR_RD,0x55, 0x8a,brightness);
	return brightness[0];
}

/* TODO #2-2 sensor function to read gain register */
/* Copy the SensorGetBrightness function (right aobve this comment),
 * paste it below this comment and rename it to SensorGetGain.
 * Change the register address accessed to 0xCC12.
 */

/*
   Update the brightness setting for the MT9M114 sensor.
 */
void
SensorSetBrightness (
        uint8_t input)
{
	SensorWrite1B(SENSOR_ADDR_WR, 0x55, 0x8a, (input&0xff));
}

/* TODO #2-3 sensor function to write gain register */
/* Copy the SensorSetBrightness function (right above),
 * paste it below this comment and rename it to SensorSetGain.
 * Change the register address accessed to 0xCC12.
 */


